@@ -12,7 +12,7 @@ from espider.response import Response
 from inspect import isgenerator
 from pprint import pformat
 from espider._utils._colorlog import ColoredFormatter
-from collections import Iterable
+from collections.abc import Iterable, Coroutine
 
 try:
     from redis import Redis
@@ -58,7 +58,7 @@ class Spider(object):
 
         self._priority_item_map = {}
         self._priority_request_map = {}
-        self._callback_priority_map = {}
+        self._priority_callback_map = {}
         self._next_priority_index = 1
 
         self._default_filter_queue = set()
@@ -69,10 +69,13 @@ class Spider(object):
         self.LOG_DATEFMT = '%Y/%m/%d %H:%M:%S'
 
         self.REQUEST_DELAY = 0
-        self.REQUEST_DETAIL = False
+        self.REQUEST_WARNING = True
         self.REQUEST_QUEUE = PriorityQueue()
         self.REQUEST_MIDDLEWARES = [self.__request_filter]
         self.REQUEST_BATCH_SIZE = 10
+        self.REQUEST_TIMEOUT = aiohttp.ClientTimeout(total=600)
+        self.REQUEST_CONNECTOR = aiohttp.TCPConnector(limit=100)
+        self.REQUEST_SESSION = aiohttp.ClientSession(connector=self.REQUEST_CONNECTOR, timeout=self.REQUEST_TIMEOUT)
 
         self.SPIDER_LOOP = asyncio.get_event_loop()
         self.SPIDER_STOP_COUNTDOWN = 3
@@ -81,27 +84,28 @@ class Spider(object):
         self.USER_AGENT_LIST = USER_AGENT_LIST
         self.RESPONSE_MIDDLEWARES = [self.__response_filter]
 
-        self.headers = None
-        self.cookies = {}
+        self.headers = self.headers if hasattr(self, 'headers') else None
+        self.cookies = self.cookies if hasattr(self, 'cookies') else {}
 
         if os.path.exists('settings.py'):
             import settings as st
             for key in self.__dict__.keys() & st.__dict__.keys():
                 self.__setattr__(key, st.__dict__.get(key))
 
-            self.headers = {'User-Agent': random.choice(self.USER_AGENT_LIST)}
-
             if 'REQUEST_HEADERS' in st.__dict__.keys():
                 self.headers = st.__dict__.get('REQUEST_HEADERS')
 
             if 'REQUEST_COOKIES' in st.__dict__.keys():
                 self.cookies = st.__dict__.get('REQUEST_COOKIES')
-        else:
-            self.headers = {'User-Agent': random.choice(USER_AGENT_LIST)}
 
         self.logger = logging.getLogger(self.name)
         self._start_time = time.time()
 
+    def start(self):
+        self.prepare()
+        self.SPIDER_LOOP.run_until_complete(self.__run())
+        self.close()
+
     @property
     def msg(self):
         return {
@@ -159,21 +163,136 @@ class Spider(object):
             else:
                 await self.__process_item(1, r)
 
-    def start(self):
-        self.prepare()
-        self.SPIDER_LOOP.run_until_complete(self.__run())
-        self.close()
-
     async def __run(self):
         """
         主函数
         """
-        self.__assert_params()
-        self.__init_logger()
-        consumer = asyncio.ensure_future(self.__downloader())
-        await self.__init_queue()
-        await self.REQUEST_QUEUE.join()
-        consumer.cancel()
+        try:
+            self.__assert_params()
+            self.__init_logger()
+        except Exception as e:
+            self.logger.exception(e)
+        else:
+            consumer = asyncio.ensure_future(self.__downloader())
+            await self.__init_queue()
+            await self.REQUEST_QUEUE.join()
+            await self.REQUEST_SESSION.close()
+            consumer.cancel()
+
+    def request(self, url=None, method=None, data=None, json=None, headers=None, cookies=None, callback=None,
+                cb_args=None, cb_kwargs=None, priority=None, allow_redirects=True, **kwargs):
+        """
+        请求创建函数
+        """
+
+        if callback is None: callback = self.parse
+        if callback.__name__ not in self._priority_callback_map.keys():
+            self._priority_callback_map[callback.__name__] = self._next_priority_index
+            self._next_priority_index += 1
+
+        if priority is None: priority = self._priority_callback_map.get(callback.__name__)
+
+        request_params = {
+            'url': url,
+            'method': method or 'GET',
+            'data': data,
+            'json': json,
+            'headers': headers,
+            'cookies': cookies,
+            'allow_redirects': allow_redirects,
+            **kwargs,
+        }
+        return priority, Request(**request_params), callback, cb_args or (), cb_kwargs or {}
+
+    async def async_request(self, priority, req, callback, *args, **kwargs):
+        """
+        异步请求
+        """
+        try:
+            # 处理请求
+            req = await self.__process_request(req)
+            if req is None: return
+
+            if self.REQUEST_DELAY: await asyncio.sleep(self.REQUEST_DELAY)
+
+            msg = self.__collect_msg(priority, req)
+            self.logger.info(msg)
+            # FIXME aiohttp.client_exceptions.ServerDisconnectedError
+            async with self.REQUEST_SESSION.request(**req.__dict__) as _resp:
+                data = await _resp.read()
+                resp = Response(_resp)
+                resp.text = data
+                resp.request = req
+                if self.REQUEST_WARNING and resp.status_code != 200:
+                    detail = pformat(req.__dict__).replace('\n', '\n\t\t')
+                    detail = re.sub('^\{', '{\n\t\t ', detail)
+                    detail = re.sub('\}$', '\n\t}', detail)
+                    msg = msg + '{}\n\t{}\n'.format(resp.status_code, detail)
+                    self.logger.warning(msg)
+
+            # 更新爬虫信息
+            self.__update_msg(priority)
+
+            # 处理响应
+            resp = await self.__process_response(resp)
+            if resp is None: return
+
+        except Exception as e:
+            self.logger.exception(msg=f"Async Request Error: {e}")
+        else:
+            # 处理回调函数
+            await self.__process_callback(priority, callback, resp, *args, **kwargs)
+
+    async def __process_request(self, req):
+
+        if not req.headers: req.headers = self.headers or {'User-Agent': random.choice(self.USER_AGENT_LIST)}
+        if isinstance(req.headers, str): req.headers = headers_to_dict(req.headers)
+        if not req.cookies: req.cookies = self.cookies
+        if isinstance(req.cookies, str): req.cookies = cookies_to_dict(req.cookies)
+        # 调用请求中间件
+        req = await self.__process_middleware(req, self.REQUEST_MIDDLEWARES)
+        if req is None:
+            self._msg_request_dropped += 1
+            return
+        return req
+
+    async def __process_response(self, resp):
+        # 调用响应中间件
+        resp = await self.__process_middleware(resp, self.RESPONSE_MIDDLEWARES)
+        if resp is None:
+            self._msg_response_dropped += 1
+            return
+        return resp
+
+    @staticmethod
+    async def __process_middleware(req, middlewares):
+        if not middlewares: return req
+        for mid in middlewares:
+            req = mid(req)
+            if isinstance(req, Coroutine): req = await req
+            if not req: return
+        return req
+
+    async def __process_callback(self, priority, callback, resp, *args, **kwargs):
+        """
+        处理回调函数
+        """
+        try:
+            result = callback(resp, *args, **kwargs)
+            if isinstance(result, Coroutine): result = await result
+        except Exception as e:
+            self.logger.exception(msg='Process Callback({}) Error: {}'.format(callback.__name__, e))
+        else:
+            if not result: return
+            if isgenerator(result):
+                for r in result:
+                    if not r: continue
+                    if isinstance(r, tuple) and isinstance(r[1], Request):
+                        self.REQUEST_QUEUE.put_nowait(r)
+                    else:
+                        await self.__process_item(priority, r)
+            else:
+                await self.__process_item(priority, result)
 
     async def __downloader(self):
         """
@@ -195,7 +314,6 @@ class Spider(object):
                     self.SPIDER_STOP_COUNTDOWN -= 1
                     if self.SPIDER_STOP_COUNTDOWN < 0:
                         self.__close_msg()
-
             else:
                 if self.REQUEST_QUEUE.empty():
                     self.SPIDER_STOP_COUNTDOWN -= 1
@@ -209,160 +327,13 @@ class Spider(object):
         if hasattr(self.REQUEST_QUEUE, '_finished'):
             self.REQUEST_QUEUE._finished.set()
 
-    def __close_msg(self):
-        self.logger.info('Close Spider - Priority: Item {} Request {}'.format(
-            self._priority_item_map,
-            self._priority_request_map,
-        ))
-        self.logger.info('Close Spider - Speed: {}'.format(
-            {k: v for k, v in self.msg.items() if 'speed' in k}
-        ))
-        self.logger.info('Close Spider - Download: {}'.format(
-            {k: v for k, v in self.msg.items() if 'speed' not in k and 'dropped' not in k}
-        ))
-        self.logger.info('Close Spider - Dropped: {}'.format(
-            {k: v for k, v in self.msg.items() if 'dropped' in k}
-        ))
-        self.logger.info('Close Spider - Request Middlewares - {}'.format(
-            [m.__name__ for m in self.REQUEST_MIDDLEWARES]
-        ))
-        self.logger.info('Close Spider - Response Middlewares - {}'.format(
-            [m.__name__ for m in self.RESPONSE_MIDDLEWARES]
-        ))
-        self.logger.info('Close Spider - Item Pipelines - {}'.format(
-            [m.__name__ for m in self.ITEM_PIPELINES]
-        ))
-
-    def __create_task(self, task_params):
+    async def __create_task(self, task_params):
         priority, req, callback, cb_args, cb_kwargs = task_params
-
-        task = self.SPIDER_LOOP.create_task(
+        return await self.SPIDER_LOOP.create_task(
             self.async_request(
                 priority, req, callback, *cb_args, **cb_kwargs
             )
         )
-        return task
-
-    def request(self, url=None, method=None, data=None, json=None, headers=None, cookies=None, callback=None,
-                cb_args=None, cb_kwargs=None, priority=None, allow_redirects=True, **kwargs):
-        """
-        请求创建函数
-        """
-
-        if callback is None: callback = self.parse
-        if callback.__name__ not in self._callback_priority_map.keys():
-            self._callback_priority_map[callback.__name__] = self._next_priority_index
-            self._next_priority_index += 1
-
-        if priority is None: priority = self._callback_priority_map.get(callback.__name__)
-        if headers is None: headers = self.headers
-        if cookies is None: cookies = self.cookies
-
-        request_params = {
-            'url': url,
-            'method': method or 'GET',
-            'data': data,
-            'json': json,
-            'headers': headers or {'User-Agent': random.choice(USER_AGENT_LIST)},
-            'cookies': cookies,
-            'allow_redirects': allow_redirects,
-            **kwargs,
-        }
-        return priority, Request(**request_params), callback, cb_args or (), cb_kwargs or {}
-
-    async def async_request(self, priority, req, callback, *args, **kwargs):
-        """
-        异步请求
-        """
-
-        try:
-            # 调用请求中间件
-            req = self.__process_middleware(req, self.REQUEST_MIDDLEWARES)
-            if req is None:
-                self._msg_request_dropped += 1
-                return
-
-            if isinstance(req.__dict__.get('cookies', {}), str):
-                req.__dict__['cookies'] = cookies_to_dict(req.__dict__.get('cookies'))
-
-            if isinstance(req.__dict__.get('headers', {}), str):
-                req.__dict__['headers'] = headers_to_dict(req.__dict__.get('cookies'))
-
-            if self.REQUEST_DELAY: await asyncio.sleep(self.REQUEST_DELAY)
-
-            async with aiohttp.request(**req.__dict__) as _resp:
-                data = await _resp.read()
-                resp = Response(_resp)
-                resp.text = data
-                resp.request = req
-
-            self._msg_total_requests += 1
-            self._msg_download_speed = self._msg_total_requests / (self._msg_runtime or 1)
-
-            if str(priority) not in self._priority_request_map.keys(): self._priority_request_map[str(priority)] = 0
-            self._priority_request_map[str(priority)] += 1
-
-            self.logger.info(self.__collect_msg(priority, req, resp))
-
-            # 调用响应中间件
-            resp = self.__process_middleware(resp, self.RESPONSE_MIDDLEWARES)
-            if resp is None:
-                self._msg_response_dropped += 1
-                return
-
-        except Exception as e:
-            self.logger.exception(msg=f"Async Request Error: {e}")
-        else:
-            await self.__process_callback(priority, callback, resp, *args, **kwargs)
-
-    @staticmethod
-    def __process_middleware(req, middlewares):
-        if not middlewares: return req
-        for mid in middlewares:
-            req = mid(req)
-            if not req: return
-        return req
-
-    def __collect_msg(self, priority, req, resp):
-        self._msg_runtime = time.time() - self._start_time
-
-        msg = '[{}] [{}/{:.2f}s/{:.2f}] {} {} {} '.format(
-            priority,
-            self._msg_total_requests,
-            self._msg_runtime,
-            self._msg_download_speed,
-            self._priority_request_map,
-            req.__dict__.get('method'),
-            req.__dict__.get('url'),
-        )
-
-        if self.REQUEST_DETAIL or resp.status_code != 200:
-            detail = pformat(req.__dict__).replace('\n', '\n\t\t')
-            detail = re.sub('^\{', '{\n\t\t ', detail)
-            detail = re.sub('\}$', '\n\t}', detail)
-            msg = msg + '{}\n\t{}\n'.format(resp.status_code, detail)
-
-        return msg
-
-    async def __process_callback(self, priority, callback, resp, *args, **kwargs):
-        """
-        处理回调函数
-        """
-        try:
-            result = callback(resp, *args, **kwargs)
-        except Exception as e:
-            self.logger.exception(msg='Process Callback({}) Error: {}'.format(callback.__name__, e))
-        else:
-            if not result: return
-            if isgenerator(result):
-                for r in result:
-                    if not r: continue
-                    if isinstance(r, tuple) and isinstance(r[1], Request):
-                        await self.REQUEST_QUEUE.put(r)
-                    else:
-                        await self.__process_item(priority, r)
-            else:
-                await self.__process_item(priority, result)
 
     async def __process_item(self, priority, item):
         """
@@ -371,18 +342,26 @@ class Spider(object):
         try:
             for pipe in self.ITEM_PIPELINES:
                 item = pipe(item)
+                if isinstance(item, Coroutine): item = await item
                 if item is None:
                     self._msg_item_dropped += 1
-                    return
         except Exception as e:
             self.logger.exception(e)
         else:
+            # 更新 Item 信息
             self._msg_items += 1
             self._msg_item_speed = self._msg_items / (self._msg_runtime or 1)
-
             if str(priority) not in self._priority_item_map.keys(): self._priority_item_map[str(priority)] = 0
             self._priority_item_map[str(priority)] += 1
 
+    def __update_msg(self, priority):
+        self._msg_runtime = time.time() - self._start_time
+        self._msg_total_requests += 1
+        self._msg_download_speed = self._msg_total_requests / (self._msg_runtime or 1)
+
+        if str(priority) not in self._priority_request_map.keys(): self._priority_request_map[str(priority)] = 0
+        self._priority_request_map[str(priority)] += 1
+
     @staticmethod
     def __split_result(result):
         if isinstance(result, (dict, list, str)):
@@ -401,7 +380,7 @@ class Spider(object):
         else:
             return result, 1
 
-    def __request_filter(self, req):
+    async def __request_filter(self, req):
         url = req.url
         args = [canonicalize_url(url)]
 
@@ -431,15 +410,52 @@ class Spider(object):
     def parse(self, response, *args, **kwargs):
         pass
 
-    def level_2(self, response, *args, **kwargs):
-        pass
-
-    def level_3(self, response, *args, **kwargs):
-        pass
-
     def __pipeline(self, item):
-        self.logger.debug(item)
+        self.logger.debug('Item: {}'.format(item))
         return item
 
     def close(self):
         pass
+
+    def __collect_msg(self, priority, req):
+
+        return '[{}] [{:.2f}] [{}/{:.2f}] [{}/{:.2f}] {} {}'.format(
+            priority,
+            self._msg_runtime,
+            self._msg_total_requests,
+            self._msg_download_speed,
+            self._msg_items,
+            self._msg_item_speed,
+            req.__dict__.get('method'),
+            req.__dict__.get('url'),
+        )
+
+    def __close_msg(self):
+        self.logger.info('-' * 100)
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Dropped', {k: v for k, v in self.msg.items() if 'dropped' in k}
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Speed', {k: v for k, v in self.msg.items() if 'speed' in k}
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Download', {k: v for k, v in self.msg.items() if 'speed' not in k and 'dropped' not in k}
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Priority Item',
+            self._priority_item_map,
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Priority Request',
+            self._priority_request_map,
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Item Pipelines', [m.__name__ for m in self.ITEM_PIPELINES]
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Request Middlewares', [m.__name__ for m in self.REQUEST_MIDDLEWARES]
+        ))
+        self.logger.info('Spider - {:21s}: {}'.format(
+            'Response Middlewares', [m.__name__ for m in self.RESPONSE_MIDDLEWARES]
+        ))
+        self.logger.info('-' * 100)
